require File.expand_path(File.dirname(__FILE__) + '/spec_helper')

describe AssetWatcher do
  before :all do
    @target_dir = File.expand_path "sample", File.dirname(__FILE__)

    @config = {
      :src_dir => @target_dir + "/src",
      :dest_dir => @target_dir + "/public",
    }

    FileUtils.makedirs @config[:src_dir]

    @coffee_file = @config[:src_dir] + "/test.coffee"
    content =<<-Coffee
class CoffeeKlass
  constructor: ->
  square: (x) -> x * x
    Coffee
    File.open(@coffee_file, 'w') { |io| io.write content }

    @haml_file = @config[:src_dir] + "/test.haml"
    content =<<-HAML
!!! 5
%html
  %head
    %title test page
  %body
    AssetWatcher test.
    HAML

    File.open(@haml_file, 'w') { |io| io.write content }
  end

  after :all do
    FileUtils.rm_rf @target_dir if File.directory? @target_dir
  end

  before do
    @watcher = AssetWatcher.new @config
  end

  context "configurations" do
    subject { @watcher }

    its(:src_dir) { should == @config[:src_dir] }
    its(:dest_dir) { should == @config[:dest_dir] }

    context "default values" do
      before do
        current_dir = `pwd`.strip
        @default_config = {
          :src_dir => current_dir + "/src",
          :dest_dir => current_dir + "/public",
        }
      end

      subject { AssetWatcher.new }
      its(:src_dir) { should == @default_config[:src_dir] }
      its(:dest_dir) { should == @default_config[:dest_dir] }
    end

    context "yaml file values" do
      before do
        @yaml_config = {
          :src_dir => "/test/src/dir",
          :dest_dir => "/test/dest/dir",
        }
        File.stub(:exist?).and_return true
        YAML.stub(:load_file).and_return @yaml_config
      end

      subject { AssetWatcher.new }
      its(:src_dir) { should == @yaml_config[:src_dir] }
      its(:dest_dir) { should == @yaml_config[:dest_dir] }
    end
  end

  describe "#target_files" do
    before do
      @dummy_file = @config[:src_dir] + "/dummy.html"

      content = '<html><body>non-target file.</body></html>'
      File.open(@dummy_file, 'w') { |io| io.write content }
    end

    it { File.should be_exist @haml_file }

    subject { @watcher.target_files }
    it { should be_include @haml_file }
    it { should be_include @coffee_file }
    it { should_not be_include @dummy_file }
  end

  describe "#destination" do
    it { @watcher.destination(@haml_file).should == @config[:dest_dir] + "/test.html" }
    it { @watcher.destination("/foo/dummy.haml").should be_nil }
    it { @watcher.destination("/foo/dummy.html").should be_nil }
  end

  describe "#compile_haml" do
    it { @watcher.compile_haml('dummy.haml').should be_false }

    it "should use Haml::Engine" do
      mock_engine = mock 'Haml Engine'
      Haml::Engine.stub!(:new).and_return mock_engine
      mock_engine.should_receive(:render).once

      @watcher.compile_haml @haml_file
    end

    it "should compile .haml file" do
      dest_file = @watcher.destination @haml_file

      @watcher.compile_haml @haml_file
      File.should be_exist dest_file

      output = File.open(dest_file, 'r') { |io| io.read }
      output.should =~ /test page/
    end
  end

  describe "#compile_coffee" do
    it { @watcher.compile_coffee('dummy.coffee').should be_false }

    it "should compile .coffee file" do
      dest_file = @watcher.destination @coffee_file

      begin
        @watcher.compile_coffee @coffee_file
        File.should be_exist dest_file

        output = File.open(dest_file, 'r') { |io| io.read }
        output.should =~ /CoffeeKlass/
      rescue => e
        puts e.message
      end
    end
  end

  describe "#compile" do
    it "should call :compile_coffee when receives .coffee file path" do
      @watcher.should_receive(:compile_coffee).with(@coffee_file).once
      @watcher.compile @coffee_file
    end
  end
end
